#define MICO_CONF_IMR
#include <CORBA.h>
#include <iostream.h>
#include "hello.h"
#include <mico/CosTrading.h>
#include <mico/CosTradingRepos.h>
#include <mico/IdentifiableObject_impl.h>

#ifdef USE_CORBA_2_1
#define CORBA_CXX_PREFIX(x) _##x
#else
#define CORBA_CXX_PREFIX(x) _cxx_##x
#endif


class Hello_impl : virtual public IdentifiableObject_impl,
		   virtual public POA_Hello,
		   virtual public POA_CosStream::Streamable
{
    char * hello_string_;
public:
    Hello_impl() { hello_string_ = (char*)NULL; };
    Hello_impl(const char * str) { hello_string_ = CORBA::string_dup(str); };
    virtual char* hello_string() { return CORBA::string_dup(hello_string_); };
    virtual void hello_string( const char* value )
	{
	    CORBA::string_free(hello_string_);
	    hello_string_ = CORBA::string_dup(value);
	};
    
    virtual CosLifeCycle::Key* external_form_id();

    virtual void externalize_to_stream( CosStream::StreamIO_ptr targetStreamIO );
    virtual void internalize_from_stream( CosStream::StreamIO_ptr sourceStreamIO, CosLifeCycle::FactoryFinder_ptr there );
    char * hello (const char *s)
    {
	if (hello_string_)
	    cout << hello_string_<<' ';
	cout << s << endl;
	return CORBA::string_dup (s);
    }
};
CosLifeCycle::Key*
Hello_impl::external_form_id()
{
    CosLifeCycle::Key* factory_id_ = new CosLifeCycle::Key();
    (*factory_id_).length(4);
    (*factory_id_)[0].id = CORBA::string_dup("IDL:omg.org/CosStream/Streamable:1.0");
    (*factory_id_)[0].kind = CORBA::string_dup("object interface");
    (*factory_id_)[1].id = CORBA::string_dup("");
    (*factory_id_)[1].kind = CORBA::string_dup("implementation equivalence class");
    (*factory_id_)[2].id = CORBA::string_dup("IDL:omg.org/Hello:1.0");
    (*factory_id_)[2].kind = CORBA::string_dup("object implementation");
    (*factory_id_)[3].id = CORBA::string_dup("StreamableFactory");
    (*factory_id_)[3].kind = CORBA::string_dup("factory interface");
    return factory_id_;
}

void
Hello_impl::externalize_to_stream( CosStream::StreamIO_ptr targetStreamIO )
{
    targetStreamIO -> write_string(this->hello_string_);
};

void
Hello_impl::internalize_from_stream( CosStream::StreamIO_ptr sourceStreamIO, CosLifeCycle::FactoryFinder_ptr there )
{
    CORBA::string_free(hello_string_);
    hello_string_ = sourceStreamIO -> read_string();
};

class HelloFactory_impl : virtual public POA_CosStream::StreamableFactory
{
public:
    HelloFactory_impl(CORBA::ORB_ptr _orb)
	: POA_CosStream::StreamableFactory()
    {
	IdentifiableObject_impl* tmp = new IdentifiableObject_impl (_orb);
    };
    virtual CosStream::Streamable_ptr create_uninitialized()
    {
	Hello_impl* hello_impl = new Hello_impl();
	Hello_ptr hello = hello_impl->_this();
//	CORBA::string_free(str);
	return hello;
    };
};

int
main (int argc, char *argv[])
{
    CORBA::ORB_var orb = CORBA::ORB_init (argc, argv, "mico-local-orb");
    CORBA::Object_var poaobj = orb->resolve_initial_references ("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow (poaobj);
    PortableServer::POAManager_var mgr = poa->the_POAManager();

    HelloFactory_impl* hello = new HelloFactory_impl(orb);
    poa->activate_object (hello);

    CORBA::Object_var tr_obj;
#ifdef HAVE_EXCEPTIONS
    try {
#endif
	tr_obj = orb -> resolve_initial_references("TradingService");
#ifdef HAVE_EXCEPTIONS
    } catch(CORBA::ORB::InvalidName&) {
	cerr << "Can`t resolve Trading Service. Abort" << endl;
	exit(1);
    }
#endif
    if (CORBA::is_nil(tr_obj))
    {
	cerr << "Can`t resolve Trading Service. Abort" << endl;
	exit(1);
    }
    CosTrading::Lookup_var trader_ = CosTrading::Lookup::_narrow (tr_obj);
    CosTrading::Register_var reg = trader_->register_if();
    if (CORBA::is_nil(reg))
    {
	cerr << "Invalid Trading Service. Abort" << endl;
	exit(1);
    }
    CORBA::Object_var obj = trader_->type_repos();
    CosTradingRepos::ServiceTypeRepository_var repo =
	CosTradingRepos::ServiceTypeRepository::_narrow(obj);
    if (CORBA::is_nil(repo))
    {
	cerr << "Invalid Trading Service. Abort" << endl;
	exit(1);
    }

    CORBA::Boolean gen_type = false;
    CosTradingRepos::ServiceTypeRepository::SpecifiedServiceTypes all_;
    all_._d(CosTradingRepos::ServiceTypeRepository::all);
    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq* list_of_type = 
	repo -> list_types(all_);
    for( CORBA::ULong i = 0;i < list_of_type->length();++i)
    {
	CosTradingRepos::ServiceTypeRepository::TypeStruct* tmp_type =
		repo -> describe_type((*list_of_type)[i]);
	if (!strcmp("StreamableFactory",(*list_of_type)[i]) &&
	    !strcmp("IDL:omg.org/CosStream/StreamableFactory:1.0",tmp_type->if_name))
	{
	    delete tmp_type;
	    break;
	}
    }
    delete list_of_type;
    if (!gen_type)
    {
	CosTradingRepos::ServiceTypeRepository::IncarnationNumber i1;
	{
	    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq super;
	    super.length( 0 );
	    CosTradingRepos::ServiceTypeRepository::PropStructSeq props;
	    props.length( 3 );
	    props[0].name = CORBA::string_dup( "ObjectInterface" );
	    props[0].value_type = CORBA::_tc_string;
	    props[0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY;
	    props[1].name = CORBA::string_dup( "ImplEquivClass" );
	    props[1].value_type = CORBA::_tc_string;
	    props[1].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
	    props[2].name = CORBA::string_dup( "ObjectImpl" );
	    props[2].value_type = CORBA::_tc_string;
	    props[2].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;

	    i1 = repo->add_type( "StreamableFactory", "IDL:omg.org/CosStream/StreamableFactory:1.0", props, super );
	}
    }
    CosTrading::OfferId id1;

    CosTrading::PropertySeq seq;
    seq.length(3);
    seq[0].name = CORBA::string_dup( "ObjectInterface" );
    seq[0].value <<= CORBA::Any::from_string( "IDL:omg.org/CosStream/Streamable:1.0", 0 );
    seq[1].name = CORBA::string_dup( "ImplEquivClass" );
    seq[1].value <<= CORBA::Any::from_string("", 0);
    seq[2].name = CORBA::string_dup( "ObjectImpl" );
    seq[2].value <<= CORBA::Any::from_string("IDL:omg.org/Hello:1.0", 0);

    id1 = reg->CORBA_CXX_PREFIX(export)(hello->_this(),"StreamableFactory", seq );

    mgr->activate ();

    orb->run ();
    return 0;
}
